/**************************************************************
 * RobotLib.h 
 *
 * Copyright (C) 2002 - Pacific Star Technologies 
 * All Rights Reserved. Unauthorized dissemination of this file
 * is strictly prohibited.
 *
 * Created: Jul 24, 2002     
 *************************************************************/

#ifndef ROBOTLIB_H_
#define ROBOTLIB_H_

#include <windows.h>
#include <ole2.h>
#include <ocidl.h>
#include <RobotDefs.h>
#include <RobotEvent.h>
#include <RobotModule.h>

typedef DWORD EventID;
typedef EventID * PEventID;

DECLARE_INTERFACE_(IRobot, IUnknown)
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  // attaches an add-on module with given CLSID 
  STDMETHOD(AddModule) (THIS_ REFCLSID) PURE;
  // detaches an add-on module
  STDMETHOD(RemoveModule) (THIS_ REFCLSID) PURE;
  // enumerates currently loaded modules
  STDMETHOD(EnumModules) (THIS_ IEnumRbtModules **) PURE; 
  // enumerates registered modules on the system
  STDMETHOD(EnumSysModules) (THIS_ IEnumRbtModules **) PURE;
  // finds an interface among the loaded modules
  STDMETHOD(FindInterface) (THIS_ REFIID iid, void **ppv) PURE;
  // gets the IRbtModule interface on a module
  STDMETHOD(GetModule) (THIS_ CLSID, void **ppv) PURE;

  // displays prop sheet with pages from all current modules
  STDMETHOD(GetPropertyPages) (THIS_ CAUUID *, IUnknown ***, UINT *) PURE;

  // registers a custom event
  STDMETHOD(RegisterEvent) (THIS_ LPCTSTR, PEventID, PEventID) PURE;
  // unregisters an event
  STDMETHOD(UnregisterEvent) (THIS_ EventID) PURE;
  // gets the event id for an event
  STDMETHOD(GetEventIDFromName) (THIS_ LPCTSTR, PEventID, PEventID) PURE;
  // gets the name for an event
  STDMETHOD(GetEventNameFromID) (THIS_ EventID, TCHAR **) PURE;
  // fires an event, causing all registered event handlers to be called
  STDMETHOD(FireEvent) (THIS_ EventID, DWORD) PURE;
  // enumerates custom events
  STDMETHOD(EnumEvents) (THIS_ IEnumRbtEvents **) PURE;
};  

DECLARE_INTERFACE_(ISpecifyActions, IUnknown) 
{
  STDMETHOD(QueryInterface) (THIS_ REFIID, void **) PURE;
  STDMETHOD_(ULONG, AddRef) (THIS) PURE;
  STDMETHOD_(ULONG, Release) (THIS) PURE;

  STDMETHOD(GetActions) (THIS_ LPTSTR **, UINT *) PURE;
  STDMETHOD(Dispatch) (THIS_ LPCTSTR) PURE;
};
#endif
