/**************************************************************
 * Module.cpp
 *
 * Copyright (C) 2002 - Pacific Star Technologies Ltd
 * All Rights Reserved. Unauthorized dissemination of this file
 * is strictly prohibited.
 *
 * Created: Aug 29, 2002 
 *************************************************************/

#include <windows.h>
#include <tchar.h>
#include <initguid.h>
#include <stdio.h>

#include "RobotGuid.h"
#include "resource.h"
#include "Module.h"
#include "Helper.h"
#include "PropPage.h"
#include "StrCast.h"

#pragma warning (disable : 4355 4244)

static const UINT ALLOC_BLOCK = 8;

// TODO: Change ModuleName to name of your module class

ModuleName::ModuleName(IUnknown *pOuter)
: module(pOuter, this), propSpec(pOuter, this), actSpec(pOuter, this)
{
  refCnt = 0;
}

ModuleName::~ModuleName() 
{
}

STDMETHODIMP
ModuleName::QueryInterface(REFIID iid, void **ppv)
{
  if (NULL == ppv) return E_POINTER;

  // TODO: Adjust according to interfaces exported by your class
  *ppv = NULL;
  if (IID_IRbtModule == iid) {
    *ppv = &module;
  } else if (IID_IRbtModuleName == iid) {
    *ppv = &module;
  } else if (IID_ISpecifyPropertyPages == iid) {
    *ppv = &propSpec;
  } else if (IID_ISpecifyActions == iid) {
    *ppv = &actSpec;
  } else if (IID_IUnknown == iid) {
    *ppv = this;
  }

  if (NULL != *ppv) {
    ((IUnknown *)*ppv)->AddRef();
    return S_OK;
  }
  return E_NOINTERFACE;
}

STDMETHODIMP_(ULONG)
ModuleName::AddRef()
{
  return ++refCnt;
}

STDMETHODIMP_(ULONG)
ModuleName::Release()
{
  ULONG cnt = --refCnt;
  if (0 == cnt) {
    refCnt++;
    delete this;
    CSvrState::state->DecObjs();
  }
  return cnt;
}

HRESULT
ModuleName::Init(IRobot *rbt)
{
  HRESULT hr;
 
  dllInst = CSvrState::state->dllInst;
  
  //TODO: Init variables here. If this function fails, Unload() must
  //TODO: be able to undo everything here

  pRobot = rbt;
  rbt->AddRef();

  hr = LoadPrereqs();
  CHECK_HR(hr);

  hr = ReadSettings();
  CHECK_HR(hr);

  // TODO: Add whatever other setup steps you need 

  return hr;
}

HRESULT
ModuleName::Unload()
{ 
  SaveSettings();

  // TODO: Release held resources

  SAFE_REL(pRobot);
  return S_OK;
}

HRESULT
ModuleName::LoadPrereqs()
{
  HRESULT hr = S_OK;

  // TODO: Make sure all dependencies are loaded and obtain a pointer to them
  return hr;
}

HRESULT
ModuleName::ReadSettings()
{
  // TODO: Read settings from registry or else where
  return S_OK;
}

HRESULT
ModuleName::SaveSettings()
{
  // TODO: Save settings to registry or else where
  return S_OK;
}

ModuleName::CoModImpl::CoModImpl(IUnknown *pOuter, ModuleName *pBack)
{
  parent = pBack;
  if (NULL == pOuter)
    outer = pBack;
  else
    outer = pOuter;
}

ModuleName::CoModImpl::~CoModImpl()
{
}

STDMETHODIMP
ModuleName::CoModImpl::QueryInterface(REFIID iid, void **ppv)
{
  return outer->QueryInterface(iid, ppv);
}

STDMETHODIMP_(ULONG)
ModuleName::CoModImpl::AddRef()
{
  return outer->AddRef();
}

STDMETHODIMP_(ULONG)
ModuleName::CoModImpl::Release()
{
  return outer->Release();
}

STDMETHODIMP
ModuleName::CoModImpl::Init(IRobot *rbt)
{
  return parent->Init(rbt);
}

STDMETHODIMP
ModuleName::CoModImpl::Unload()
{
  return parent->Unload();
}

STDMETHODIMP
ModuleName::CoModImpl::GetName(LPTSTR name, UINT sz)
{
  lstrcpyn(name, moduleName, sz);
  return S_OK;
}

STDMETHODIMP
ModuleName::CoModImpl::GetCLSID(CLSID *clsid)
{
  if (NULL == clsid) return E_POINTER;
  *clsid = CLSID_ModuleName;
  return S_OK;
}

// TODO: Implement property accessors here

ModuleName::CoPropSpecImpl::CoPropSpecImpl(IUnknown *pOuter, ModuleName *pBack)
{
  parent = pBack;
  if (NULL == pOuter)
    outer = pBack;
  else
    outer = pOuter;
}

ModuleName::CoPropSpecImpl::~CoPropSpecImpl()
{
}

STDMETHODIMP
ModuleName::CoPropSpecImpl::QueryInterface(REFIID iid, void **ppv)
{
  return outer->QueryInterface(iid, ppv);
}

STDMETHODIMP_(ULONG)
ModuleName::CoPropSpecImpl::AddRef()
{
  return outer->AddRef();
}

STDMETHODIMP_(ULONG)
ModuleName::CoPropSpecImpl::Release()
{
  return outer->Release();
}

STDMETHODIMP
ModuleName::CoPropSpecImpl::GetPages(CAUUID *pages)
{
  // TODO: Adjust to number & CLSID of property pages in your module

  pages->cElems = 1;
  pages->pElems = (GUID *)CoTaskMemAlloc(sizeof(UUID)*1);
  if (pages->pElems == NULL) return E_OUTOFMEMORY;
  pages->pElems[0] = CLSID_PropPage1;
  return S_OK;
}

ModuleName::CoActSpecImpl::CoActSpecImpl(IUnknown *pOuter, ModuleName *pBack)
{
  parent = pBack;
  if (NULL == pOuter)
    outer = pBack;
  else
    outer = pOuter;
}

ModuleName::CoActSpecImpl::~CoActSpecImpl()
{
}

STDMETHODIMP
ModuleName::CoActSpecImpl::QueryInterface(REFIID iid, void **ppv)
{
  return outer->QueryInterface(iid, ppv);
}

STDMETHODIMP_(ULONG)
ModuleName::CoActSpecImpl::AddRef()
{
  return outer->AddRef();
}

STDMETHODIMP_(ULONG)
ModuleName::CoActSpecImpl::Release()
{
  return outer->Release();
}

STDMETHODIMP
ModuleName::CoActSpecImpl::GetActions(LPTSTR **actions, UINT *cnt)
{
  // TODO: Adjust to actions to be added to menu

  *actions = (LPTSTR *)CoTaskMemAlloc(sizeof(LPTSTR *)*3);
  if (NULL == *actions) return E_OUTOFMEMORY;
  (*actions)[0] = CoTaskStrDup(_T("Action 1"));
  (*actions)[1] = CoTaskStrDup(_T("Action 2"));
  if ((*actions)[0] == NULL || (*actions)[1] == NULL) {
    CoTaskMemFree(*actions);
    return E_OUTOFMEMORY;
  }
  *cnt = 2;
  return S_OK;
}

STDMETHODIMP
ModuleName::CoActSpecImpl::Dispatch(LPCTSTR action)
{
  // TODO: Dispatch action based on str command

  if (!lstrcmp(action, _T("Action 1"))) {
    // do something
    return S_OK;
  } else if (!lstrcmp(action, _T("Action 2"))) {
    // do something else
    return S_OK;
  } else {
    return E_INVALIDARG;
  }
}
